%% PBmodel.m
%% Purpose: create graphics like the one in P+B Fig 7.1, based on my_f0modXX.m
%%          tuned for "nomi1p"
%% Mafuyu Kitahara
%% 6/28/2000	created

%% prepare matrices for work space
	%% number of steps
	l = 1; % number of H* scales
	m = 1; % number of L% scales
	n = 1; % number of H*L slopes

	%% slope step size
	sl = 30; 

	%% initialize matrices
	f0raw_bkup = f0raw;		
	len = length(f0raw);
	tt=1:len;
	f0mixes = zeros(l,len,m,n);
	f0news = zeros(l,len,m,n);
	f0interp = zeros(l,len,m,n);
	f0approx = f0raw;
	f0new = f0raw;
	decline = zeros(len);

% find start and end point of non-zero f0raw
	nz = find (f0raw~=0); 	f0st = f0raw(nz(1));
	nzL = length(nz);	f0en = f0raw(nz(nzL));		

% marks
	mk0t = round(mk0x(1,1));	f0mk0 = f0raw(mk0t);
	mk1t = round(mk1x(1,1));	f0mk1 = f0raw(mk1t);
	mk2t = round(mk2x(1,1));	f0mk2 = f0raw(mk2t);
	mk3t = round(mk3x(1,1));	f0mk3 = f0raw(mk3t);
	mk4t = round(mk4x(1,1));	f0mk4 = f0raw(mk4t);
	mk5t = round(mk5x(1,1));	f0mk5 = f0raw(mk5t);
	mk6t = round(mk6x(1,1));	f0mk6 = f0raw(mk6t);
	mk7t = round(mk7x(1,1));	f0mk7 = f0raw(mk7t);
	mk8t = round(mk8x(1,1));	f0mk8 = f0raw(mk8t);

% Mid point of marks
	mk12t = round((mk1t+mk2t)/2);	f0mk12 = f0raw(mk12t);
	mk23t = round((mk2t+mk3t)/2);	f0mk23 = f0raw(mk23t);
	mk34t = round((mk3t+mk4t)/2);	f0mk34 = f0raw(mk34t);
	mk45t = round((mk4t+mk5t)/2);	f0mk45 = f0raw(mk45t);
	mk56t = round((mk5t+mk6t)/2);	f0mk56 = f0raw(mk56t);
	mk67t = round((mk6t+mk7t)/2);	f0mk67 = f0raw(mk67t);
	mk78t = round((mk7t+mk8t)/2);	f0mk78 = f0raw(mk78t);

% determine window size from mora length
	moras = [mk2t-mk0t mk4t-mk2t mk8t-mk6t];
	window = round(mean(moras));

% create jitter
	f0sp = sprandn(1,len,0.5); % use sparse random number at density 0.3
	f0noise = 15*full(f0sp);   % convert sparse into full+enhance noisiness

	%% a little bit of smoothing 
	f0jitt = zeros(len);
	ww = 20;
	half = round(ww/2);
	hh = ones(ww,1)/ww;	
	f0jitt = conv(hh,f0noise);  	% convolution
	f0jitt = f0jitt(half+1:len+half); % centering


%-------------------------------------------------------
% modify f0 values
%-------------------------------------------------------

	% H* scale
	for i = 1:l
		f0n3 =  f0mk4+18; % ad hoc values to fit the result with the original
		f0n4 =  f0mk4+18;

	% L% scale
	for j = 1:m
		f0n8 = f0mk8+34; % ad hoc values to fit the result with the original
	
	% H*L slope
	for k = 1:n
		% seg1 from H* to L
		% calculate slopes from H* to L then determine L
		seg1 = -(f0n4-f0n8)/(1+(k-1)*sl); % larger k -> shallower
		b1 = f0n4-seg1*(mk4t+window);	  % intercept
		y1 = seg1*tt+b1;

		dif = abs(y1-f0n8);		% L equal to new L%
		Lt = find(dif==min(dif));

		% check monotonicity
		if Lt == mk4t
			Lt = Lt+1; 	% sneaky! 
			k=n;		% do for loop only once
		end

		% check redundant slopes in zero region
		%if f0raw(Lt)==0
		%	k=n;		% do for loop only once
		%end

		% vectorize
		xx = [1 mk0t-5 mk0t mk2t-5 mk2t mk4t mk4t+5 mk8t len];
		yy = [f0mk0 f0mk0 f0mk1+5 f0mk1+5 f0n3 f0n4 f0n8 f0n8 f0n8];

		% declination
		f0dec = f0mk0-f0mk8;

		xd = [1 mk0t mk8t len];
		yd = [0 0 -f0dec -f0dec];
		decline = interp1(xd,yd,tt,'linear');

		% check if seg1 slope is smaller than decline
		%if abs((decline(mk4t)-decline(Lt))/(Lt-mk4t)) >= abs(seg1)
		%	break
		%end

		% linear new f0 contour
		f0new = interp1(xx,yy,tt,'linear');
		f0interp(i,:,j,k) = f0new;	% store interpolated tones

		% add declination
		f0new = f0new + decline;
		f0news(i,:,j,k) = f0new;	% store declinated tones

		%% smoothing (taken from P+B and DSP FIRST, p133)

		half = round(window/2);
		hh = ones(window,1)/window;	
		f0sm = conv(hh,f0new);  	% convolution
		f0sm = f0sm(1:len); % length adjustment
%		f0sm = f0sm(half+1:len+half); % centering
		f0sms(i,:,j,k) = f0sm;

		%% jitter and devoicing
		f0mixes(i,:,j,k) = f0sm+f0jitt; 
		z = find (f0raw==0); % zero of f0raw
		f0mixes(i,z,j,k) = 0;

	end
	end
	end
%%% plot

%% new figure
aa=figure;
%set(aa,'PaperPosition',[0.3 0.25 7 3.5]); % for portrait
orient landscape
set(aa,'Position',[40 130 1200 800]); %left, bottom, w, h

%% Tone sequence

subplot('Position',[0.05 0.75 0.9 0.23]); %left, bottom, w, h
plot(tt,f0raw,'--')
	set(gca,'Ylim',[70 180]);
	set(gca,'Xlim',[0 700]);
	set(gca,'xtick',[]);
	title([fname]);
%	xlabel('time (ms)');
	ylabel('Frequency (Hz)');
for i=1:l
  for j=1:m
    for k=1:n
	line(tt,f0interp(i,:,j,k),'Color','red');
    end
  end
end

% add markers
line(mk0x, mky, 'Color','black');
line(mk1x, mky, 'Color','black');
line(mk2x, mky, 'Color','black');
line(mk3x, mky, 'Color','black');
line(mk4x, mky, 'Color','black');
line(mk5x, mky, 'Color','black');
line(mk6x, mky, 'Color','black');
line(mk7x, mky, 'Color','black');
line(mk8x, mky, 'Color','black');

%% declined  

subplot('Position',[0.05 0.51 0.9 0.23 ]); %left, bottom, w, h
plot(tt,f0raw,'--');
	set(gca,'Ylim',[70 180]);
	set(gca,'Xlim',[0 700]);
	set(gca,'xtick',[]);
for i=1:l
  for j=1:m
    for k=1:n
	line(tt,f0news(i,:,j,k),'Color','red');
    end
  end
end

% add markers
line(mk0x, mky, 'Color','black');
line(mk1x, mky, 'Color','black');
line(mk2x, mky, 'Color','black');
line(mk3x, mky, 'Color','black');
line(mk4x, mky, 'Color','black');
line(mk5x, mky, 'Color','black');
line(mk6x, mky, 'Color','black');
line(mk7x, mky, 'Color','black');
line(mk8x, mky, 'Color','black');

%% smoothed

subplot('Position',[0.05 0.27 0.9 0.23]); %left, bottom, w, h
plot(tt,f0raw,'--')
	set(gca,'Ylim',[70 180]);
	set(gca,'Xlim',[0 700]);
	set(gca,'xtick',[]);
for i=1:l
  for j=1:m
    for k=1:n
	line(tt,f0sms(i,:,j,k),'Color','red');
    end
  end
end

% add markers
line(mk0x, mky, 'Color','black');
line(mk1x, mky, 'Color','black');
line(mk2x, mky, 'Color','black');
line(mk3x, mky, 'Color','black');
line(mk4x, mky, 'Color','black');
line(mk5x, mky, 'Color','black');
line(mk6x, mky, 'Color','black');
line(mk7x, mky, 'Color','black');
line(mk8x, mky, 'Color','black');

%%% f0 jittered and devoiced

subplot('Position',[0.05 0.03 0.9 0.23]); %left, bottom, w, h
plot(tt,f0raw,'--')
	set(gca,'Ylim',[70 180]);
	set(gca,'Xlim',[0 700]);
for i=1:l
  for j=1:m
    for k=1:n
	line(tt,f0mixes(i,:,j,k),'Color','red');
    end
  end
end

% add markers
line(mk0x, mky, 'Color','black');
line(mk1x, mky, 'Color','black');
line(mk2x, mky, 'Color','black');
line(mk3x, mky, 'Color','black');
line(mk4x, mky, 'Color','black');
line(mk5x, mky, 'Color','black');
line(mk6x, mky, 'Color','black');
line(mk7x, mky, 'Color','black');
line(mk8x, mky, 'Color','black');





