%% my_f0mod02dev_try.m
%% Purpose: For creating stimuli continuum in 0-2 opposition, 
%% starting from accent type 0, for 6-mora phrases
%% Mafuyu Kitahara
%% 8/04/2000	% limit final L% range always lower than initial L%
		% redesigned for 6-mora phrases.

%% prepare matrices for work space
	%% number of steps
	l = 6; % number of H* scales
	m = 6; % number of L% scales
	n = 6; % number of phrasal H scales

	%% slope step size (fixed to 100)
	sl = 0.423; 	% in Hz/ms obtained from average (voiced accented 
			% 24 tokens)

	f0mean = avf0;	% use mean as the reference (not in P+B sense tho)

	%% approximation line smoothing window size
	awindow = 60;	% should be the same as in my_diffslope.m

	% final lowering parameters
	% lower f0 after phrasal H dependeing on dur b/w
	% mark(3:11), f_rate=0.8 (arbitrary)

	fldur = marks(11)-marks(3); % final lowering span
	fldrop = 20; % final lowering scale = 20Hz (arbitrary)
	flrate = 0.8;


	%% initialize matrices
	f0raw_bkup = f0raw;		
	len = length(f0raw);
	tt=1:len;
	f0mixes = zeros(l,len,m,n);
	f0news = zeros(l,len,m,n);
	f0interp = zeros(l,len,m,n);
	f0new = f0raw;
	decline = zeros(len);

	% matrices for saving parameters
	lx = 9; %% change if the length of xx and yy changes
	Ptime = zeros(l,lx,m,n);	% to store Parameters of time 
	Pnews = zeros(l,lx,m,n);	% to store Parameters in f0news
	Pmixes = zeros(l,lx,m,n);	% to store Parameters in f0mixes
	Pother = zeros(l,lx,m,n);	% to store other parameters

% find start and end point of non-zero f0raw
	nz = find (f0raw~=0); 	f0st = f0raw(nz(1));
	nzL = length(nz);	f0en = f0raw(nz(nzL));		

% f0 at marks (assume "marks" vector exists in the workspace)

	% check the length of the nonzero marks
	L=length(find(marks));
	for i=1:L-1
		f0marks(i) = f0raw(marks(i));
	end

%---------------------------------------------------------------------
% Approximation
%---------------------------------------------------------------------

%% filtered f0raw for approx, add "f0lead" for centering process

	f0lead = zeros(1,awindow);
	f0fill = cat(2,f0lead,f0raw);
	z = find (f0fill==0); 	% zero of f0fill
	f0fill(z) = avf0;	% fill devoiced part by mean of 
				% non-zero part of f0raw ("avf0" taken from 
				% my_MBE.m)

	ahalf = round(awindow/2);
	hh = ones(awindow,1)/awindow;	
	f0filt = conv(hh,f0fill);  	% convolution
	f0approx = f0filt(awindow+ahalf+1:len+awindow+ahalf); 	% centering
								% + adjust lead

%% get diff b/w approx and raw
	f0diff = f0raw-f0approx;

%-------------------------------------------------------
% modify f0 values
%-------------------------------------------------------

% 0-2 modification: marks(6:7) <- H*

	% H* scale
	for i = 1:l
		f0n6 = f0mean*2^((i-1)/15); % larger i -> higher H*

		% final lowering for H*
		f0n6 = f0n6-(fldrop*flrate*(marks(6)-marks(3))/fldur);		

	% L% scale
	for j = 1:m
		f0n11 = f0mean*2^(-(j-1)/15); % larger j -> lower L%

		% final lowering for L% tone
		f0n11 = f0n11-(fldrop*(1-flrate)*(marks(11)-marks(3))/fldur);

	% phrasal H scale
	for k = 1:n  
		f0nH = f0mean*2^(-(k-1)/15); % larger k -> lower phrasal H
		f0i = f0nH*2^(-1/12); 	% initial L% = 1ST lower than phrasal H

		% no rising toward the end
		if f0nH < f0n11 
			break
		end

	% curve from H*L to L%

		% final lowering for accent L tone (actually raising a bit)
		%f0nL = f0n11+(fldrop*(1-flrate)*(marks(11)-Lt(1))/fldur);

		% seg1 from H* to L
		% calculate slopes from H* to L then determine L
		seg1 = -sl; 
		b1 = f0n6-seg1*marks(7);% intercept
		y1 = seg1*tt+b1;	% new line

		dif = abs(y1-f0n11);
		Lt = find(dif==min(dif));

		%%% always make zero-slope version (unaccented case)
		if (i==1 & j==1 & k==1)
			Lt = marks(11)-1;
			f0n6 = f0n11;
		end

		%%% check monotonicity
		if Lt <= marks(7)
			break
			%Lt = marks(7)+1; 	% sneaky! 
			%k=n;		% do for loop only once
		end

		%%% check if the end of the slope is beyond utterance end
		if Lt >= marks(11)
			break
		end

		%%% check redundant slopes in zero region
		%if f0raw(Lt)==0
		%	i=l;
		%end


		% vectorize
xx = [1   marks(1) marks(2) marks(3) marks(6) marks(7) Lt(1) marks(11) len];
yy = [f0i f0i      f0nH     f0nH     f0n6     f0n6     f0n11   f0n11     f0n11];


		% declination
		% simply difference between f0marks(1) and f0marks(11)
		f0dec2 = f0marks(11)-f0marks(1);

		% or 10Hz/sec (P+B 1988)
		f0dec = 10*(marks(11)-marks(1))/1000;

		xd = [1 marks(1) marks(11) len];
		yd = [f0dec f0dec 0 0];
		decline = interp1(xd,yd,tt,'linear');

		% linear new f0 contour
		f0new = interp1(xx,yy,tt,'linear');
		f0interp(i,:,j,k) = f0new;	% store interpolated tones

		% add declination
		f0new = f0new + decline;
		f0news(i,:,j,k) = f0new;	% store declinated tones

		% store parameters
		Ptime(i,:,j,k) = xx;
		Pnews(i,:,j,k) = yy;

		%% smoothing (taken from P+B and DSP FIRST, p133)
		%window = mk4t-mk3t;		% vowel-size window
		window = 16;			% arbitrary
		half = round(window/2);
		hh = ones(window,1)/window;	
		f0sm = conv(hh,f0new);  	% convolution
		f0sm = f0sm(half+1:len+half); % centering

		f0mixes(i,:,j,k) = f0sm+f0diff; % do convolution
		%f0mixes(i,:,j,k) = f0news(i,:,j,k)+f0diff; % OR skip convolution

		%% taking care of zeros
		z = find (f0raw==0); % zero of f0raw
		f0mixes(i,z,j,k) = 0;

		%% check if output contains no signal
		z=find(f0mixes(i,:,j,k)==0);
		if length(z)==length(tt) 
		  break
		end

		Pmixes(i,:,j,k) = f0mixes(i,xx,j,k);
		OtherP = [f0dec f0dec2 awindow window sl l m n f0mean];% new 6/27
		Pother(i,:,j,k) = OtherP;
	end
	end
	end
%%% plot

%% new figure
	aa=figure;
	%set(aa,'PaperPosition',[0.3 0.25 7 3.5]); % for portrait
	orient landscape
	set(aa,'Position',[40 130 1200 800]); %left, bottom, w, h

%% f0 original and approx

subplot('Position',[0.05 0.71 0.9 0.26]); %left, bottom, w, h
	plot(tt,f0raw,tt,f0approx,'--');
	set(gca,'Ylim',[80 200]);
	set(gca,'Xlim',[0 850]);
	set(gca,'xtick',[]);
	title([fname]);
%	xlabel('time (ms)');
	ylabel('Frequency (Hz)');

	% draw markers
	mky = [-50 5000];
	z = find(marks>0);
	lastmark = length(marks(z));

	for i=1:lastmark-1
		line([marks(i) marks(i)], mky, 'Color','black');
	end

%% f0 difference and declination
	z = zeros(1,len);	
subplot('Position',[0.05 0.57 0.9 0.13 ]); %left, bottom, w, h
	plot(tt,f0diff,'color','black');
	line(tt,decline,'color','magenta');
	line(1:len,z,'color','red','linestyle','-');
	set(gca,'Ylim',[-15 45]);
	set(gca,'Xlim',[0 850]);
	set(gca,'xtick',[]);

	% draw markers
	for i=1:lastmark-1
		line([marks(i) marks(i)], mky, 'Color','black');
	end

%% f0 interpolated

subplot('Position',[0.05 0.30 0.9 0.26]); %left, bottom, w, h
plot(tt,f0raw,':')
	set(gca,'Ylim',[80 200]);
	set(gca,'Xlim',[0 850]);
	set(gca,'xtick',[]);
	for i=1:l
	  for j=1:m
	    for k=1:n
		line(tt,f0interp(i,:,j,k),'Color','red');
	    end
	  end
	end

	% draw markers
	for i=1:lastmark-1
		line([marks(i) marks(i)], mky, 'Color','black');
	end

%%% f0 synth

subplot('Position',[0.05 0.03 0.9 0.26]); %left, bottom, w, h
	plot(tt,f0raw,':')
	set(gca,'Ylim',[80 200]);
	set(gca,'Xlim',[0 850]);
	for i=1:l
	  for j=1:m
	    for k=1:n
		line(tt,f0mixes(i,:,j,k),'Color','red');
	    end
	  end
	end

	% mark zero-slope with green
	line(tt,f0mixes(1,:,1,1),'Color','green');

	% draw markers
	for i=1:lastmark-1
		line([marks(i) marks(i)], mky, 'Color','black');
	end

%% new plot just for synth

%% new figure
	bb=figure;
	%set(bb,'PaperPosition',[0.3 0.25 7 3.5]); % for portrait
	orient landscape
	set(bb,'Position',[40 130 1200 800]); %left, bottom, w, h

subplot('Position',[0.05 0.67 0.9 0.3]); %left, bottom, w, h
	plot(tt,f0raw,':')
	set(gca,'Ylim',[80 200]);
	set(gca,'Xlim',[0 850]);
	title([fname]);
%	xlabel('time (ms)');
	ylabel('Frequency (Hz)');
	for i=1:l
	  for j=1:m
	    for k=1:1
		line(tt,f0mixes(i,:,j,k),'Color','red');
	    end
	  end
	end

	% draw markers
	for i=1:lastmark-1
		line([marks(i) marks(i)], mky, 'Color','black');
	end

subplot('Position',[0.05 0.35 0.9 0.3]); %left, bottom, w, h
	plot(tt,f0raw,':')
	set(gca,'Ylim',[80 200]);
	set(gca,'Xlim',[0 850]);
	ylabel('Frequency (Hz)');
	for i=1:l
	  for j=1:m
	    for k=2:2
		line(tt,f0mixes(i,:,j,k),'Color','red');
	    end
	  end
	end

	% draw markers
	for i=1:lastmark-1
		line([marks(i) marks(i)], mky, 'Color','black');
	end
subplot('Position',[0.05 0.03 0.9 0.3]); %left, bottom, w, h
	plot(tt,f0raw,':')
	set(gca,'Ylim',[80 200]);
	set(gca,'Xlim',[0 850]);
	ylabel('Frequency (Hz)');
	for i=1:l
	  for j=1:m
	    for k=6:6
		line(tt,f0mixes(i,:,j,k),'Color','red');
	    end
	  end
	end

	% draw markers
	for i=1:lastmark-1
		line([marks(i) marks(i)], mky, 'Color','black');
	end

