%% my_f0mod02dev-try.m
%% Purpose: For creating stimuli continuum in 0-2 opposition, 
%% starting from accent type 0
%% Mafuyu Kitahara
%% 6/2/2000	created
%% 6/22/2000 	major update, converged with my_f0mod12e.m
%% 6/23/2000	bug fix, new plotting schema 
%% 6/26/2000	approximation by filtered f0raw
%% 8/02/2000	trial version for new things
		% single slope, variable phrasal H
		% use f0mean as the reference point
		% differet smoothing window for approx
%% 8/04/2000	% limit final L% range always lower than initial L%


%% prepare matrices for work space
	%% number of steps
	l = 6; % number of H* scales
	m = 6; % number of L% scales
	n = 6; % number of phrasal H scales

	%% slope step size (fixed to 100)
	sl = 0.3; % in Hz/ms

	%% TRIAL: change them dynamically according to new H*
	%% preset values for tones
	%f0H = 140;	% arbitrary preset value for phrasal H 
	%f0i = exp(log(2)*(log2(f0H)-3/12)); % initial L%, 3ST below H, 
	%f0nH = exp(log(2)*(log2(f0H)+3/12)); % H* 3ST above H
	%f0Lper = f0i;	% final L% 

	f0mean = avf0;	% use mean as the reference (not in P+B sense tho)

	%% approximation line smoothing window size
	awindow = 60;	% should be the same as in my_diffslope.m

	%% initialize matrices
	f0raw_bkup = f0raw;		
	len = length(f0raw);
	tt=1:len;
	f0mixes = zeros(l,len,m,n);
	f0news = zeros(l,len,m,n);
	f0interp = zeros(l,len,m,n);
	f0new = f0raw;
	decline = zeros(len);

	% matrices for saving parameters
	lx = 9; %% change if the length of xx and yy changes
	Ptime = zeros(l,lx,m,n);	% to store Parameters of time 
	Pnews = zeros(l,lx,m,n);	% to store Parameters in f0news
	Pmixes = zeros(l,lx,m,n);	% to store Parameters in f0mixes
	Pother = zeros(l,lx,m,n);	% to store other parameters

% find start and end point of non-zero f0raw
	nz = find (f0raw~=0); 	f0st = f0raw(nz(1));
	nzL = length(nz);	f0en = f0raw(nz(nzL));		

% marks
	mk0t = round(mk0x(1,1));	f0mk0 = f0raw(mk0t);
	mk1t = round(mk1x(1,1));	f0mk1 = f0raw(mk1t);
	mk2t = round(mk2x(1,1));	f0mk2 = f0raw(mk2t);
	mk3t = round(mk3x(1,1));	f0mk3 = f0raw(mk3t);
	mk4t = round(mk4x(1,1));	f0mk4 = f0raw(mk4t);
	mk5t = round(mk5x(1,1));	f0mk5 = f0raw(mk5t);
	mk6t = round(mk6x(1,1));	f0mk6 = f0raw(mk6t);
	mk7t = round(mk7x(1,1));	f0mk7 = f0raw(mk7t);
	mk8t = round(mk8x(1,1));	f0mk8 = f0raw(mk8t);

% Mid point of marks
	mk12t = round((mk1t+mk2t)/2);	f0mk12 = f0raw(mk12t);
	mk23t = round((mk2t+mk3t)/2);	f0mk23 = f0raw(mk23t);
	mk34t = round((mk3t+mk4t)/2);	f0mk34 = f0raw(mk34t);
	mk45t = round((mk4t+mk5t)/2);	f0mk45 = f0raw(mk45t);
	mk56t = round((mk5t+mk6t)/2);	f0mk56 = f0raw(mk56t);
	mk67t = round((mk6t+mk7t)/2);	f0mk67 = f0raw(mk67t);
	mk78t = round((mk7t+mk8t)/2);	f0mk78 = f0raw(mk78t);

%---------------------------------------------------------------------
% Approximation
%---------------------------------------------------------------------

%% filtered f0raw for approx, add "f0lead" for centering process

	f0lead = zeros(1,awindow);
	f0fill = cat(2,f0lead,f0raw);
	z = find (f0fill==0); 	% zero of f0fill
	f0fill(z) = avf0;	% fill devoiced part by mean of 
				% non-zero part of f0raw ("avf0" taken from 
				% my_MBE.m)

	ahalf = round(awindow/2);
	hh = ones(awindow,1)/awindow;	
	f0filt = conv(hh,f0fill);  	% convolution
	f0approx = f0filt(awindow+ahalf+1:len+awindow+ahalf); 	% centering
								% + adjust lead

%% get diff b/w approx and raw
	f0diff = f0raw-f0approx;

%-------------------------------------------------------
% modify f0 values
%-------------------------------------------------------

	% H* scale
	for i = 1:l
		f0n5 = f0mean*2^((i-1)/12); % larger i -> higher H*
		f0n6 = f0n5;

	% L% scale
	for j = 1:m
		f0n8 = f0mean*2^(-(j-1)/12); % larger j -> lower L%

	% phrasal H scale
	for k = 1:n  
		f0nH = f0mean*2^(-(k-1)/12); % larger k -> lower phrasal H
		f0i = f0nH*2^(-1/12); 	% initial L% = 1ST lower than phrasal H

		% seg1 from H* to L
		% calculate slopes from H* to L then determine L
		seg1 = -sl; 
		b1 = f0n6-seg1*mk6t;	% intercept
		y1 = seg1*tt+b1;	% new line

		dif = abs(y1-f0n8);
		Lt = find(dif==min(dif));

		%%% always make zero-slope version (unaccented case)
		if (i==1 & j==1 & k==1)
			Lt = mk8t-1;
			f0n8 = f0n6;
		end

		%%% check monotonicity
		if Lt == mk6t
			Lt = Lt+1; 	% sneaky! 
			k=n;		% do for loop only once
		end

		%%% check if the end of the slope is beyond utterance end
		if Lt >= mk8t
			break
		end

		%%% check redundant slopes in zero region
		if f0raw(Lt)==0
			i=l;
		end


		% vectorize
		xx = [1 mk0t mk1t mk2t mk5t mk6t Lt(1) mk8t len];
		yy = [f0i f0i f0nH f0nH f0n5 f0n6 f0n8 f0n8 f0n8];

		% declination
		% slope is fixed based on t'Hart etal 1990, p128
		% D(slope ST/s) = -11/(t(sec)+1.5)
		decl = 11*len/(mk8t-mk0t+1500);	% duration dependent
		%f0dec = exp(log(2)*(log2(f0Lper)+decl/12))-f0Lper;

		% or simply difference between f0mk0 and f0mk8
		f0dec2 = f0mk0-f0mk8;

		% or 10Hz/sec (P+B 1988)
		f0dec = 10*len/1000;

		xd = [1 mk1t mk8t len];
		yd = [f0dec f0dec 0 0];
		decline = interp1(xd,yd,tt,'linear');

		% check if seg1 slope is smaller than decline
		%if abs((decline(mk4t)-decline(Lt))/(Lt-mk4t)) >= abs(seg1)
		%	break
		%end

		% linear new f0 contour
		f0new = interp1(xx,yy,tt,'linear');
		f0interp(i,:,j,k) = f0new;	% store interpolated tones

		% add declination
		f0new = f0new + decline;
		f0news(i,:,j,k) = f0new;	% store declinated tones

		% store parameters
		Ptime(i,:,j,k) = xx;
		Pnews(i,:,j,k) = yy;

		%% smoothing (taken from P+B and DSP FIRST, p133)
		%window = mk4t-mk3t;		% vowel-size window
		window = 16;			% arbitrary
		half = round(window/2);
		hh = ones(window,1)/window;	
		f0sm = conv(hh,f0new);  	% convolution
		f0sm = f0sm(half+1:len+half); % centering

		f0mixes(i,:,j,k) = f0sm+f0diff; % do convolution
		%f0mixes(i,:,j,k) = f0news(i,:,j,k)+f0diff; % OR skip convolution

		%% taking care of zeros
		z = find (f0raw==0); % zero of f0raw
		f0mixes(i,z,j,k) = 0;

		%% check if output contains no signal
		z=find(f0mixes(i,:,j,k)==0);
		if length(z)==length(tt) 
		  break
		end

		Pmixes(i,:,j,k) = f0mixes(i,xx,j,k);
		OtherP = [f0dec f0dec2 awindow window sl l m n f0mean];% new 6/27
		Pother(i,:,j,k) = OtherP;
	end
	end
	end
%%% plot

%% new figure
	aa=figure;
	%set(aa,'PaperPosition',[0.3 0.25 7 3.5]); % for portrait
	orient landscape
	set(aa,'Position',[40 130 1200 800]); %left, bottom, w, h

%% f0 original and approx

subplot('Position',[0.05 0.71 0.9 0.26]); %left, bottom, w, h
	plot(tt,f0raw,tt,f0approx,'--');
	set(gca,'Ylim',[80 200]);
	set(gca,'Xlim',[0 850]);
	set(gca,'xtick',[]);
	title([fname]);
%	xlabel('time (ms)');
	ylabel('Frequency (Hz)');

	% draw markers
	mky = [-50 5000];
	z = find(marks>0);
	lastmark = length(marks(z));

	for i=1:lastmark-1
		line([marks(i) marks(i)], mky, 'Color','black');
	end

%% f0 difference and declination
	z = zeros(1,len);	
subplot('Position',[0.05 0.57 0.9 0.13 ]); %left, bottom, w, h
	plot(tt,f0diff,'color','black');
	line(tt,decline,'color','magenta');
	line(1:len,z,'color','red','linestyle','-');
	set(gca,'Ylim',[-15 45]);
	set(gca,'Xlim',[0 850]);
	set(gca,'xtick',[]);

	% draw markers
	for i=1:lastmark-1
		line([marks(i) marks(i)], mky, 'Color','black');
	end

%% f0 interpolated

subplot('Position',[0.05 0.30 0.9 0.26]); %left, bottom, w, h
plot(tt,f0raw,':')
	set(gca,'Ylim',[80 200]);
	set(gca,'Xlim',[0 850]);
	set(gca,'xtick',[]);
	for i=1:l
	  for j=1:m
	    for k=1:n
		line(tt,f0interp(i,:,j,k),'Color','red');
	    end
	  end
	end

	% draw markers
	for i=1:lastmark-1
		line([marks(i) marks(i)], mky, 'Color','black');
	end

%%% f0 synth

subplot('Position',[0.05 0.03 0.9 0.26]); %left, bottom, w, h
	plot(tt,f0raw,':')
	set(gca,'Ylim',[80 200]);
	set(gca,'Xlim',[0 850]);
	for i=1:l
	  for j=1:m
	    for k=1:n
		line(tt,f0mixes(i,:,j,k),'Color','red');
	    end
	  end
	end

	% mark zero-slope with green
	line(tt,f0mixes(1,:,1,1),'Color','green');

	% draw markers
	for i=1:lastmark-1
		line([marks(i) marks(i)], mky, 'Color','black');
	end


