%% my_f0modA.m
%% Purpose: For creating stimuli continuum in H* alignment dimension
%%          starting from C (variable)
%% Caution: Depends on labeling scheme done in my_plotB
%% Mafuyu Kitahara
%% 12/08/2002
		
%%%%%% User variables %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Originally accented vowel
acc = 'V3s'

%% number of steps
l = 7; % number of H* align, if Vn is accented, the span
       % of H* align is from Vn-1 to Vn+1

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%% Constants %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% number of steps
	m = 1; % L% scale fixed at L5
	n = 1; % number of phrasal H scales

%% step size
	step = 15; %(12 = 1ST, 15 = 4/5ST, 18 = 3/4ST)

%% slope 
	sl = 0.423; 	% in Hz/ms obtained from average (voiced accented 
			% 24 tokens)

	f0mean = avf0;	% use mean as the reference (not in P+B sense tho)

%% approximation line smoothing window size
	awindow = 60;	% should be the same as in my_diffslope.m

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%---------------------------------------------------------------------
% Preprocess
%---------------------------------------------------------------------
orgacc = cell(1,1); % create an empty cell array
orgacc(1,1) = {acc}; % set value from User varaiable
orgacci=strmatch(orgacc(1,1),labels); % get the index


% final lowering parameters
% lower f0 after phrasal H dependeing on dur b/w
% mark(orgacci:lenmark), f_rate=0.8 (arbitrary)

	fldur = marks(lenmark)-marks(orgacci); % final lowering span
	fldrop = 20; % final lowering scale = 20Hz (arbitrary)
	flrate = 0.8;

	%% initialize matrices
	f0raw_bkup = f0raw;		
	len = length(f0raw);
	tt=1:len;
	f0mixes = zeros(l,len,m,n);
	f0news = zeros(l,len,m,n);
	f0interp = zeros(l,len,m,n);
	f0new = f0raw;
	decline = zeros(len);

	% matrices for saving parameters
	lx = 9; %% change if the length of xx and yy changes
	Ptime = zeros(l,lx,m,n);	% to store Parameters of time 
	Pnews = zeros(l,lx,m,n);	% to store Parameters in f0news
	Pmixes = zeros(l,lx,m,n);	% to store Parameters in f0mixes
	Pother = zeros(l,lx,m,n);	% to store other parameters

% find start and end point of non-zero f0raw
	nz = find (f0raw~=0); 	f0st = f0raw(nz(1));
	nzL = length(nz);	f0en = f0raw(nz(nzL));		

% f0 at marks (assume "marks" vector exists in the workspace)

	% check the length of the nonzero marks
	L=length(find(marks));
	for i=1:L-1
		f0marks(i) = f0raw(marks(i));
	end

%---------------------------------------------------------------------
% Approximation
%---------------------------------------------------------------------

%% filtered f0raw for approx, add "f0lead" for centering process

	f0lead = zeros(1,awindow);
	f0fill = cat(2,f0lead,f0raw);
	z = find (f0fill==0); 	% zero of f0fill
	f0fill(z) = avf0;	% fill devoiced part by mean of 
				% non-zero part of f0raw ("avf0" taken from 
				% my_MBE.m)

	ahalf = round(awindow/2);
	hh = ones(awindow,1)/awindow;	
	f0filt = conv(hh,f0fill);  	% convolution
	f0approx = f0filt(awindow+ahalf+1:len+awindow+ahalf); 	% centering
								% + adjust lead

%% get diff b/w approx and raw
	f0diff = f0raw-f0approx;

%-------------------------------------------------------
% modify f0 values
%-------------------------------------------------------

% 1-2 modification: marks(4:5) -- marks (6:7) <- H*

	% H* align
	tsteps = (marks(9)-marks(5))/l; % increment relative to duration
	for i = 1:l
		mk4n = round(marks(4)+tsteps*(i-1)); % larger i -> later H*
		mk5n = round(marks(5)+tsteps*(i-1));
		f0n4 = f0mean*2^((5)/step); 

	% L% scale
	for j = m:m
		f0n11 = f0mean*2^(-(j-1)/step); 

		% final lowering for L% tone
		f0n11 = f0n11-(fldrop*(1-flrate)*(marks(11)-marks(3))/fldur);

	% phrasal H scale
	for k = 1:n  
		f0nH = f0mean*2^(-(k-1)/step); % larger k -> lower phrasal H
		f0i = f0nH*2^(-1/12); 	% initial L% = 1ST lower than phrasal H

		% no rising toward the end
		if f0nH < f0n11 
			break
		end

	% curve from H*L to L%

		% final lowering for accent L tone (actually raising a bit)
		%f0nL = f0n11+(fldrop*(1-flrate)*(marks(11)-Lt(1))/fldur);

		% seg1 from H* to L
		% calculate slopes from H* to L then determine L
		seg1 = -sl; 
		b1 = f0n4-seg1*mk5n;% intercept
		y1 = seg1*tt+b1;	% new line

		dif = abs(y1-f0n11);
		Lt = find(dif==min(dif));

		%%% always make zero-slope version (unaccented case)
		if (i==1 & j==1 & k==1)
			Lt = marks(11)-1;
			f0n4 = f0n11;
		end

		%%% check monotonicity
		%if Lt <= marks(5)
		%	break
			%Lt = marks(7)+1; 	% sneaky! 
			%k=n;		% do for loop only once
		%end

		%%% check if the end of the slope is beyond utterance end
		if Lt >= marks(11)
			break
		end

		%%% check redundant slopes in zero region
		%if f0raw(Lt)==0
		%	i=l;
		%end


		% vectorize
xx = [1   marks(1) marks(2) marks(3) mk4n     mk5n     Lt(1) marks(11) len];
yy = [f0i f0i      f0nH     f0nH     f0n4     f0n4     f0n11   f0n11     f0n11];


		% declination
		% simply difference between f0marks(1) and f0marks(11)
		f0dec2 = f0marks(11)-f0marks(1);

		% or 10Hz/sec (P+B 1988)
		f0dec = 10*(marks(11)-marks(1))/1000;

		xd = [1 marks(1) marks(11) len];
		yd = [f0dec f0dec 0 0];
		decline = interp1(xd,yd,tt,'linear');

		% linear new f0 contour
		f0new = interp1(xx,yy,tt,'linear');
		f0interp(i,:,j,k) = f0new;	% store interpolated tones

		% add declination
		f0new = f0new + decline;
		f0news(i,:,j,k) = f0new;	% store declinated tones

		% store parameters
		Ptime(i,:,j,k) = xx;
		Pnews(i,:,j,k) = yy;

		%% smoothing (taken from P+B and DSP FIRST, p133)
		%window = mk4t-mk3t;		% vowel-size window
		window = 16;			% arbitrary
		half = round(window/2);
		hh = ones(window,1)/window;	
		f0sm = conv(hh,f0new);  	% convolution
		f0sm = f0sm(half+1:len+half); % centering

		f0mixes(i,:,j,k) = f0sm+f0diff; % do convolution
		%f0mixes(i,:,j,k) = f0news(i,:,j,k)+f0diff; % OR skip convolution

		%% taking care of zeros
		z = find (f0raw==0); % zero of f0raw
		f0mixes(i,z,j,k) = 0;

		%% check if output contains no signal
		z=find(f0mixes(i,:,j,k)==0);
		if length(z)==length(tt) 
		  break
		end

		Pmixes(i,:,j,k) = f0mixes(i,xx,j,k);
		OtherP = [f0dec f0dec2 awindow window sl l m n f0mean];% new 6/27
		Pother(i,:,j,k) = OtherP;
	end
	end
	end
%%% plot

%% new figure
	aa=figure;
	%set(aa,'PaperPosition',[0.3 0.25 7 3.5]); % for portrait
	orient landscape
	set(aa,'Position',[40 130 1200 800]); %left, bottom, w, h

%% f0 original and approx

subplot('Position',[0.05 0.71 0.9 0.26]); %left, bottom, w, h
	plot(tt,f0raw,tt,f0approx,'--');
	set(gca,'Ylim',[80 200]);
	set(gca,'Xlim',[0 850]);
	set(gca,'xtick',[]);
	title([fname]);
%	xlabel('time (ms)');
	ylabel('Frequency (Hz)');

	% draw markers
	mky = [-50 5000];
	z = find(marks>0);
	lastmark = length(marks(z));

	for i=1:lastmark-1
		line([marks(i) marks(i)], mky, 'Color','black');
	end

%% f0 difference and declination
	z = zeros(1,len);	
subplot('Position',[0.05 0.57 0.9 0.13 ]); %left, bottom, w, h
	plot(tt,f0diff,'color','black');
	line(tt,decline,'color','magenta');
	line(1:len,z,'color','red','linestyle','-');
	set(gca,'Ylim',[-15 45]);
	set(gca,'Xlim',[0 850]);
	set(gca,'xtick',[]);

	% draw markers
	for i=1:lastmark-1
		line([marks(i) marks(i)], mky, 'Color','black');
	end

%% f0 interpolated

subplot('Position',[0.05 0.30 0.9 0.26]); %left, bottom, w, h
plot(tt,f0raw,':')
	set(gca,'Ylim',[80 200]);
	set(gca,'Xlim',[0 850]);
	set(gca,'xtick',[]);
	for i=1:l
	  for j=1:m
	    for k=1:n
		line(tt,f0interp(i,:,j,k),'Color','red');
	    end
	  end
	end

	% draw markers
	for i=1:lastmark-1
		line([marks(i) marks(i)], mky, 'Color','black');
	end

%%% f0 synth

subplot('Position',[0.05 0.03 0.9 0.26]); %left, bottom, w, h
	plot(tt,f0raw,':')
	set(gca,'Ylim',[80 200]);
	set(gca,'Xlim',[0 850]);
	for i=1:l
	  for j=1:m
	    for k=1:n
		line(tt,f0mixes(i,:,j,k),'Color','red');
	    end
	  end
	end

	% mark zero-slope with green
	line(tt,f0mixes(1,:,1,1),'Color','green');

	% draw markers
	for i=1:lastmark-1
		line([marks(i) marks(i)], mky, 'Color','black');
	end

%% new plot just for synth

%% new figure
	bb=figure;
	%set(bb,'PaperPosition',[0.3 0.25 7 3.5]); % for portrait
	orient landscape
	set(bb,'Position',[40 130 1200 800]); %left, bottom, w, h

subplot('Position',[0.05 0.67 0.9 0.3]); %left, bottom, w, h
	plot(tt,f0raw,':')
	set(gca,'Ylim',[80 200]);
	set(gca,'Xlim',[0 850]);
	title([fname]);
%	xlabel('time (ms)');
	ylabel('Frequency (Hz)');
	for i=1:l
	  for j=1:m
	    for k=1:1
		line(tt,f0mixes(i,:,j,k),'Color','red');
	    end
	  end
	end

	% draw markers
	for i=1:lastmark-1
		line([marks(i) marks(i)], mky, 'Color','black');
	end

subplot('Position',[0.05 0.35 0.9 0.3]); %left, bottom, w, h
	plot(tt,f0raw,':')
	set(gca,'Ylim',[80 200]);
	set(gca,'Xlim',[0 850]);
	ylabel('Frequency (Hz)');
	for i=1:l
	  for j=1:m
	    for k=2:2
		line(tt,f0mixes(i,:,j,k),'Color','red');
	    end
	  end
	end

	% draw markers
	for i=1:lastmark-1
		line([marks(i) marks(i)], mky, 'Color','black');
	end
subplot('Position',[0.05 0.03 0.9 0.3]); %left, bottom, w, h
	plot(tt,f0raw,':')
	set(gca,'Ylim',[80 200]);
	set(gca,'Xlim',[0 850]);
	ylabel('Frequency (Hz)');
	for i=1:l
	  for j=1:m
	    for k=4:4
		line(tt,f0mixes(i,:,j,k),'Color','red');
	    end
	  end
	end

	% draw markers
	for i=1:lastmark-1
		line([marks(i) marks(i)], mky, 'Color','black');
	end

