% my_f0peak.m
% Show spectrogram and F0, mark region and get Two f0 local maxima
% tuned for P+B sentences to get high-line and reference-line
% Cf. Chapter 7
 
% Hit return when all marks are done.

% Mafuyu Kitahara
% 8/01/2000

%-----------------------------------------------------
% Initialize common variables
%-----------------------------------------------------
	mky = [-50 5000];
	mkyb = [0.5 1];

	%% initialize matrices
	len = length(f0raw);
	marks = zeros(1,len);
	labels = cell(1,len); % create a cell array

%-----------------------------------------------------
% Figure parameters
%-----------------------------------------------------
	aa=figure;
	orient landscape;
	%set(aa,'PaperPosition',[0.3 0.25 7 3.5]); % for portrait
	set(aa,'Position',[40 330 1200 600]); % size on screen by pixel

%-----------------------------------------------------
% Spectrogram part (comparison)
%-----------------------------------------------------

	% nsgram (Equal resolution spectrum)
	mxil=max(max(dB(nsgram+0.001)));
	[nsy,nsx]=size(nsgram);
 	sgrh = subplot('Position',[0.05 0.525 0.9 0.435 ]); %left, bottom, w, h
     	imagesc((0:nsx-1)*shiftm,(0:nsy-1)/nsy*fs/2, ...
	max(dB(nsgram+0.001),mxil-50));
	axis('xy');colormap(1-gray);
	set(gca,'xtick',[]);
	set(gca,'Ylim',[0 5000]); 
	title([fname]);
%	xlabel('time (ms)');
	ylabel('Frequency (Hz)');

%-----------------------------------------------------     
% F0 trajectory part
%-----------------------------------------------------
	f0h = subplot('Position',[0.05 0.1 0.9 0.38 ]);
    	plot(tt*shiftm,f0t,'r');grid on; 
	set(gca,'xaxislocation','top');
	set(gca,'Ylim',[70 200]);
	if ~isnan(avf0)
	  axis([1 max(tt)*shiftm ...
	      min(avf0/sqrt(2),0.95*min(f0raw(f0raw>0)))  ...
		  max(avf0*sqrt(2),1.05*max(f0raw(f0raw>0)))]);
        end;
%	xlabel('time (ms)');
	ylabel('F0 (Hz)');

	% add label field
	labh = subplot('Position',[0.05 0.01 0.9 0.09]);grid on;
		set(gca,'xtick',[]);
		set(gca,'ytick',[]);
		axis([1 nsx 0 1]);

%-----------------------------------------------------
% Check existing .lab file
%-----------------------------------------------------
	% label filename
	base = strtok(fname,'.'); % take part of input 
				  % filename before ".wav" ext
	labfname = strcat(base,'.lab') % reuse in the File IO part

	% check
	labfiles = dir;		% create structure array
	labfiles = [labfiles.name];	% only filenames
	if isempty(findstr(labfname,labfiles));	% proceed 

%-----------------------------------------------------
% let user to mark N points 
%-----------------------------------------------------

	disp('Left click at the point you want to mark and hit return once');
	disp('Then, input label in the command window');
	disp('Hit return in the command window to finish');

for i = 1:len % max number of marks = duration of the input in msec
	mk = ginput(len);
		mkx = [mk(1) mk(1)];
		marks(i) = round(mk(1));

		a = input('label? (hit return when done) ','s');
		if isempty(a)
        	   break
		else
	           text(mkx(1),0.3,a,'HorizontalAlignment','right','FontSize',16)
		   labels(i) = {a};   % assign the label to cell array (use {}).
		   i=i+1;
		end

	% draw lines
	axes(f0h);	line(mkx, mky);
	axes(sgrh);	line(mkx, mky);
	axes(labh);	line(mkx,mkyb);
end

% to show time axis labels, "f0h" handle should be current. 
axes(f0h);axes(sgrh);

%-----------------------------------------------------
% File OUTPUT part 
%-----------------------------------------------------
	
	% check the size of the matrices
	L=length(find(marks));

  	fid = fopen(labfname,'w');
		for ii = 1:L-1
			fprintf(fid,'%s\t',labels{ii});
			fprintf(fid,'%4.0f\n',marks(ii));
		end
   	fclose(fid); 

%-----------------------------------------------------
% check part end, use existing lab files and plot
%-----------------------------------------------------
	else 
		disp('Existing labels are shown. ');
		[labels,marks]=textread(labfname,'%s %f')
		lenmark = length(marks);

for i = 1:lenmark % max number of marks = duration of the input in msec
	mkx = [marks(i) marks(i)];
   	text(mkx(1),0.3,labels(i),'HorizontalAlignment','right','FontSize',16)
	i=i+1;

	% draw lines
	axes(f0h);	line(mkx, mky);
	axes(sgrh);	line(mkx, mky);
	axes(labh);	line(mkx,mkyb);
end

% to show time axis labels, "f0h" handle should be current. 
axes(f0h);axes(sgrh);


%-------------------------------------------------------
	end % end of "if" in checking existing lab file part
 

%-----------------------------------------------------
% get F0 peaks between marks1-2 and marks3-4 and draw lines
%-----------------------------------------------------

f0peaks = zeros(2,2); % 2x2 matrix, row for time, column for f0 
	
f0peaks(1,1)=marks(1)+...
             find(f0raw(marks(1):marks(2))==max(f0raw(marks(1):marks(2))))-1;
f0peaks(1,2)=marks(3)+...
             find(f0raw(marks(3):marks(4))==max(f0raw(marks(3):marks(4))))-1;
f0peaks(2,1) = f0raw(f0peaks(1,1));
f0peaks(2,2) = f0raw(f0peaks(1,2));

	py1 = [f0peaks(2,1)-15 f0peaks(2,1)+10];
	py2 = [f0peaks(2,2)-15 f0peaks(2,2)+10];
	px1 = [f0peaks(1,1) f0peaks(1,1)];
	px2 = [f0peaks(1,2) f0peaks(1,2)];
	axes(f0h);
	line(px1, py1,'LineWidth',1.5,'Color','black');
	line(px2, py2,'LineWidth',1.5,'Color','black');
        text(px1(1),py1(1)-10,'H_1',...
          'FontSize',16,'FontWeight','bold','HorizontalAlignment','center');
        text(px2(1),py2(1)-10,'H_2',...
          'FontSize',16,'FontWeight','bold','HorizontalAlignment','center');

%-----------------------------------------------------
% write to peakfile
%-----------------------------------------------------

	peakfname = strcat(base,'.peak') 
	colhead = char('H1','H2');
	colhead = cellstr(colhead);
  	fid = fopen(peakfname,'w');
		for ii = 1:2
		fprintf(fid,'\n%s\t',colhead{ii});
		fprintf(fid,'%4.0f',f0peaks(:,ii));
		end
   	fclose(fid); 

	% output format  
	%
	%	label	time(ms) f0(Hz)
	%	H1	345	 167
	%	H2	611	 123
	%



