% my_plotB.m 
% Show spectrogram and F0, prompt user to label N points
% Hit return when all marks are done.

% Mafuyu Kitahara
% 5/14/2000 	created
% 5/24      	minor modification: add another marker
% 5/31		minor modification: move axis labels
% 7/31/2000	major update: mark N marks, store labels and marks in a
%		.lab file
% 12/08/2002    adjusted for 1024x768 screen

%-----------------------------------------------------
% Initialize common variables
%-----------------------------------------------------
	mky = [-50 5000];
	mkyb = [0.5 1];

	%% initialize matrices
	len = length(f0raw);
	marks = zeros(1,len);
	f0marks = zeros(1,len);
	labels = cell(1,len); % create a cell array

%-----------------------------------------------------
% Figure parameters
%-----------------------------------------------------
	aa=figure;
	orient landscape;
	%set(aa,'PaperPosition',[0.3 0.25 7 3.5]); % for portrait
	set(aa,'Position',[10 10 1000 500]); % size on screen by pixel

%-----------------------------------------------------
% Spectrogram part
%-----------------------------------------------------
	mxil=max(max(dB(nsgram+0.001)));
	[nsy,nsx]=size(nsgram);
 	sgrh = subplot('Position',[0.05 0.525 0.9 0.435 ]); %left, bottom, w, h
     	imagesc((0:nsx-1)*shiftm,(0:nsy-1)/nsy*fs/2, ...
	max(dB(nsgram+0.001),mxil-50));
	axis('xy');colormap(1-gray);
	set(gca,'xtick',[]);
	set(gca,'Ylim',[0 5000]); 
	title([fname]);
%	xlabel('time (ms)');
	ylabel('Frequency (Hz)');

%-----------------------------------------------------     
% F0 trajectory part
%-----------------------------------------------------
	f0h = subplot('Position',[0.05 0.1 0.9 0.38 ]);
    	plot(tt*shiftm,f0t,'r');grid on; 
	set(gca,'xaxislocation','top');
	set(gca,'Ylim',[70 200]);
	if ~isnan(avf0)
	  axis([1 max(tt)*shiftm ...
	      min(avf0/sqrt(2),0.95*min(f0raw(f0raw>0)))  ...
		  max(avf0*sqrt(2),1.05*max(f0raw(f0raw>0)))]);
        end;
%	xlabel('time (ms)');
	ylabel('F0 (Hz)');

% add label field
	labh = subplot('Position',[0.05 0.01 0.9 0.09]);grid on;
		set(gca,'xtick',[]);
		set(gca,'ytick',[]);
		axis([1 nsx 0 1]);

%-----------------------------------------------------
% Check existing .lab file
%-----------------------------------------------------
	% label filename
	base = strtok(fname,'.'); % take part of input 
				  %filename before ".wav" ext
	labfname = strcat(base,'.lab') % reuse in the File IO part

	% check
	labfiles = dir;		% create structure array
	labfiles = [labfiles.name];	% only filenames
	if isempty(findstr(labfname,labfiles));	% proceed 

%-----------------------------------------------------
% let user to mark N points 
%-----------------------------------------------------

	disp('Left click at the point you want to mark and hit return once');
	disp('Then, input label in the command window');
	disp('Hit return in the command window to finish');

for i = 1:len % max number of marks = duration of the input in msec
	mk = ginput(len);
		mkx = [mk(1) mk(1)];
		marks(i) = round(mk(1));

		a = input('label? (hit return when done) ','s');
		if isempty(a)
        	   break
		else
	           text(mkx(1),0.3,a,'HorizontalAlignment','right','FontSize',16)
		   labels(i) = {a};   % assign the label to cell array (use {}).
%		   i=i+1;
		end

	% draw lines
	axes(f0h);	line(mkx, mky);
	axes(sgrh);	line(mkx, mky);
	axes(labh);	line(mkx,mkyb);
end

% to show time axis labels, "f0h" handle should be current. 
	axes(f0h);axes(sgrh);

% get f0 values at each mark

	% check the length of the nonzero marks
	L=length(find(marks));
	for i=1:L-1
		f0marks(i) = f0raw(marks(i));
	end

% convert to older variable names (for backward compatibility to old my_f0mod)

 mk0x = [marks(1) marks(1)];
 mk1x = [marks(2) marks(2)];
 mk2x = [marks(3) marks(3)];
 mk3x = [marks(4) marks(4)];
 mk4x = [marks(5) marks(5)];
 mk5x = [marks(6) marks(6)];
 mk6x = [marks(7) marks(7)];
 mk7x = [marks(8) marks(8)];
 mk8x = [marks(9) marks(9)];
 mk9x = [marks(10) marks(10)];
 mk10x = [marks(11) marks(11)];

%-----------------------------------------------------
% File OUTPUT part 
%-----------------------------------------------------
	
  	fid = fopen(labfname,'w');
		for ii = 1:L-1 % L specified at "get f0 values" part
			fprintf(fid,'%s\t',labels{ii});
			fprintf(fid,'%4.0f\t',marks(ii));
			fprintf(fid,'%4.0f\n',f0marks(ii));
		end
   	fclose(fid); 

%-----------------------------------------------------
% check part end, use existing lab files and plot
%-----------------------------------------------------
	else 
		disp('Existing labels are shown. ');
		[labels,marks,f0marks]=textread(labfname,'%s %f %f')
		lenmark = length(marks);

for i = 1:lenmark 
	mkx = [marks(i) marks(i)];
   	text(mkx(1),0.3,labels(i),'HorizontalAlignment','right','FontSize',16)
%	i=i+1;

	% draw lines
	axes(f0h);	line(mkx, mky);
	axes(sgrh);	line(mkx, mky);
	axes(labh);	line(mkx,mkyb);
end

% to show time axis labels, "f0h" handle should be current. 
axes(f0h);axes(sgrh);

% convert to older variable names (for backward compatibility to my_f0mod)

 mk0x = [marks(1) marks(1)];
 mk1x = [marks(2) marks(2)];
 mk2x = [marks(3) marks(3)];
 mk3x = [marks(4) marks(4)];
 mk4x = [marks(5) marks(5)];
 mk5x = [marks(6) marks(6)];
 mk6x = [marks(7) marks(7)];
 mk7x = [marks(8) marks(8)];
 mk8x = [marks(9) marks(9)];
 mk9x = [marks(10) marks(10)];
 mk10x = [marks(11) marks(11)];

%-------------------------------------------------------
	end % end of "if" in checking existing lab file part
 









