%% my_f0mod01.m
%% Purpose: For creating stimuli continuum in 0-1 opposition, 
%% starting from accent type 0 
%% Mafuyu Kitahara
%% 6/2/2000	created
%% 6/14/2000	modified 

%% new figure
figure

%% prepare matrices for work space
	%% modify m and n if you want. 
	l = 3; % number of H* scales
	m = 4; % number of L% scales
	n = 4; % number of H*L slopes

	%% preset values for tones
	f0i = 105;	% phrase initial f0 at the end of [a] in [ano]
	f0H = 130;	% phrasal H and the baseline for H*
	f0Lper = 90;	% L%

	f0raw_bkup = f0raw;		
	f0l = length(f0raw);
	tt=1:f0l;
	f0mixes = zeros(l,f0l,m,n);
	f0news = zeros(l,f0l,m,n);
	f0approx = f0raw;
	f0new = f0raw;

	Ptime = zeros(l,8,m,n);		% to store Parameters of time 
	Pnews = zeros(l,8,m,n);		% to store Parameters in f0news
	Pmixes = zeros(l,8,m,n);	% to store Parameters in f0mixes

% find start point of non-zero f0raw
	nz = find (f0raw~=0); 

% Index number of the marks
	mk0t = round(mk0x(1,1));
	mk1t = round(mk1x(1,1));
	mk2t = round(mk2x(1,1));
	mk3t = round(mk3x(1,1));
	mk4t = round(mk4x(1,1));
	mk5t = round(mk5x(1,1));
	mk6t = round(mk6x(1,1));
	mk7t = round(mk7x(1,1));
	mk8t = round(mk8x(1,1));

% Mid point of marks
	mk12t = round((mk1t+mk2t)/2);
	mk23t = round((mk2t+mk3t)/2);
	mk34t = round((mk3t+mk4t)/2);
	mk45t = round((mk4t+mk5t)/2);
	mk56t = round((mk5t+mk6t)/2);
	mk67t = round((mk6t+mk7t)/2);
	mk78t = round((mk7t+mk8t)/2);

% f0 values at marks
	f0nz = f0raw(nz(1));
	f0mk0 = f0raw(mk0t);
	f0mk1 = f0raw(mk1t);
	f0mk12 = f0raw(mk12t);
	f0mk2 = f0raw(mk2t);
	f0mk3 = f0raw(mk3t);
	f0mk34 = f0raw(mk34t);
	f0mk4 = f0raw(mk4t);
	f0mk45 = f0raw(mk45t);
	f0mk5 = f0raw(mk5t);
	f0mk56 = f0raw(mk56t);
	f0mk6 = f0raw(mk6t);
	f0mk67 = f0raw(mk67t);
	f0mk7 = f0raw(mk7t);
	f0mk78 = f0raw(mk78t);
	f0mk8 = f0raw(mk8t);		

%---------------------------------------------------------------------
% Approximation
%---------------------------------------------------------------------

% vectorize
xa = [nz(1) mk0t mk1t mk12t mk2t mk3t mk34t ...
        mk4t mk5t mk56t mk6t mk7t mk78t mk8t f0l];
ya = [f0nz f0mk0 f0mk1 f0mk12 f0mk2 ...
      f0mk3 f0mk34 f0mk4 f0mk5 f0mk56 ...
      f0mk6 f0mk7 f0mk78 f0mk8 f0mk8-20];

% taking care of zeros
	z = find (ya==0); % for devoiced cases
	xa(z) = '';	  % empty xa
	ya(z) = '';	  % empty ya

%% spline approximation
f0approx = spline(xa,ya,tt);

%%% get diff b/w approx and raw
f0raw_bkup(1:mk0t)=f0approx(1:mk0t);	
f0raw_bkup(mk8t:f0l)=f0approx(mk8t:f0l);	
%f0diff = f0raw_bkup-f0approx;			%edge process

f0diff = f0raw-f0approx;			%without edgeprocess

%-------------------------------------------------------
% modify f0 values
%-------------------------------------------------------

	% H* scale
	for i = 1:l
		f0n34 = exp(log(2)*(log2(f0H)+(i-1)/12)); %new f0 incr by ST

	% L% scale
	for j = 1:m
		f0n8 = exp(log(2)*(log2(f0Lper)+j/12)); % new f0 incr by ST
	
	% H*L slope
	for k = 1:n
		% seg1 from H* to L
		f0nH = exp(log(2)*(log2(f0H)+1/12)); %1ST from f0H
		seg1 = -(f0nH-f0H)/(k*10); % new slopes 
		b1 = f0n34-seg1*mk34t;
		y1 = seg1*tt+b1;

		% seg2 from L to L%
		%L-L% slope is fixed based on t'Hart etal 1990, p128
		% D(slope ST/s) = -11/(t(sec)+1.5)
		
		D = 11*1000/(mk8t+1500);
		f0intcep = exp(log(2)*(log2(f0n8)+D/12));

		seg2 = -(f0intcep-f0n8)/mk8t;
		y2 = seg2*tt+f0intcep;
		
		y1y2 = abs(y1-y2);
		Lt = find(y1y2==min(y1y2));
		f0L = y1(Lt);

	% check monotonicity
		if f0n8 >= f0L | f0L >= f0n34
			break
		end

	% vectorize
		xx = [1 mk0t mk1t mk12t mk34t Lt mk8t f0l];
		yy = [f0i-15 f0i f0i+10 f0H f0n34 f0L f0n8 f0Lper-10];

		Ptime(i,:,j,k) = xx;
		Pnews(i,:,j,k) = yy;

		%% linear new f0 contour
		f0new = interp1(xx,yy,tt,'linear');

		%% smoothing (taken from P+B and DSP FIRST, p133)
		hh = ones(20,1)/20;		% windowsize = 20 (ad hoc)
		f0sm = conv(hh,f0new);  	% convolution
		f0sms = f0sm(1,11:f0l+10);	% centeralize

		f0news(i,:,j,k) = f0sms;		% do convolution
		%f0news(i,:,j,k) = f0new;		% OR skip convolution

		f0mixes(i,:,j,k) = f0news(i,:,j,k)+f0diff;

		%% taking care of zeros
		z = find (f0raw==0); % zero of f0raw
		f0mixes(i,z,j,k) = 0;
		Pmixes(i,:,j,k) = f0mixes(i,xx,j,k);
	end
	end
	end
%%% plot

%subplot(211)
%plot(tt,f0raw,tt,f0diff,tt,f0approx);
%	set(gca,'Ylim',[-50 200]);
%subplot(212)
subplot(111)
plot(tt,f0mixes(1,:,1,1),tt,f0raw,tt,f0diff)
	set(gca,'Ylim',[-50 200]);
	line(tt,f0news(1,:,1,1),'Color','red');
	line(tt,f0news(l,:,m,n),'Color','red');
for i=1:l
  for j=1:m
    for k=1:n
	line(tt,f0mixes(i,:,j,k));
    end
  end
end

% add markers
line(mk0x, mky, 'Color','black');
line(mk1x, mky, 'Color','black');
line(mk2x, mky, 'Color','black');
line(mk3x, mky, 'Color','black');
line(mk4x, mky, 'Color','black');
line(mk5x, mky, 'Color','black');
line(mk6x, mky, 'Color','black');
line(mk7x, mky, 'Color','black');
line(mk8x, mky, 'Color','black');










